@@ -1,7 +1,19 @@
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
+# vim:fenc=utf-8
+#
+# Copyright © 2014 uralbash <root@uralbash.ru>
+#
+# Distributed under terms of the MIT license.
+
+"""
+CREATE, READ, DELETE, UPDATE actions for SQLAlchemy models
+"""
 import inspect
 import transaction
-from sacrud.utils import (
+
+from webhelpers.paginate import Page
+from sacrud.common.sa_helpers import (
     check_type,
     get_pk,
     get_relations,
@@ -10,7 +22,7 @@ from sacrud.utils import (
 prefix = 'crud'
 
 
-def index(session, table, order_by=None):
+def list(session, table, paginator=None, order_by=None):
     """
     Return a list of table rows.
 
@@ -19,13 +31,16 @@ def index(session, table, order_by=None):
         - `session`: DBSession.
         - `table`: table instance.
         - `order_by`: name ordered row.
+        - `paginator`: see sacrud.common.paginator.get_paginator.
     """
-    col = [c for c in table.__table__.columns]
+    col = [c for c in getattr(table, 'sacrud_list_col', table.__table__.columns)]
     pk_name = get_pk(table)
     query = session.query(table)
     if order_by:
         query = query.order_by(order_by)
     row = query.all()
+    if paginator:
+        row = Page(row, **paginator)
     if hasattr(table, '__mapper_args__'):
         mapper_args = table.__mapper_args__
     else:
@@ -66,7 +81,7 @@ def create(session, table, request=''):
         return
 
     pk_name = get_pk(table)
-    col = [c for c in table.__table__.columns]
+    col = [c for c in getattr(table, 'sacrud_detail_col', table.__table__.columns)]
     return {'pk': pk_name,
             'col': col,
             'table': table,
@@ -85,7 +100,7 @@ def read(session, table, pk):
     """
     pk_name = get_pk(table)
     obj = session.query(table).filter(getattr(table, pk_name) == pk).one()
-    col = [c for c in table.__table__.columns]
+    col = [c for c in getattr(table, 'sacrud_list_col', table.__table__.columns)]
     return {'obj': obj,
             'pk': pk_name,
             'col': col,
@@ -123,7 +138,7 @@ def update(session, table, pk, request=''):
         session.add(obj)
         transaction.commit()
         return
-
+    col_list = [c for c in getattr(table, 'sacrud_detail_col', table.__table__.columns)]
     return {'obj': obj,
             'pk': pk_name,
             'col': col_list,
