@@ -8,56 +8,23 @@ from pyramid.httpexceptions import HTTPFound
 
 from sacrud import action
 from sacrud.pyramid_ext import DBSession
-
-
-def breadcrumbs(tname,  view, id=None):
-    bc = {}
-    bc['sa_list'] = [{'name': 'Home', 'visible': True,
-                      'view': 'sa_home',
-                      'param': {'table': tname}},
-                     {'name': tname, 'visible': True,
-                      'view': 'sa_list',
-                      'param': {'table': tname}}]
-    bc['sa_create'] = bc['sa_list'][:]
-    bc['sa_create'].append({'name': 'create',
-                            'visible': False,
-                            'view': 'sa_list',
-                            'param': {'table': tname}})
-    bc['sa_read'] = bc['sa_list'][:]
-    bc['sa_read'].append({'name': id,
-                          'visible': False,
-                          'view': 'sa_list',
-                          'param': {'table': tname}})
-    bc['sa_update'] = bc['sa_read']
-    bc['sa_union'] = bc['sa_list'][:]
-    bc['sa_union'].append({'name': 'union',
-                           'visible': False,
-                           'view': 'sa_list',
-                           'param': {'table': tname}})
-
-    return bc[view]
-
-
-def get_config(request, name):
-    settings = request.registry.settings
-    if 'sacrud_models' in settings:
-        message = 'WARNING: Use "sacrud.models" key setting instead "sacrud_models !!!' +\
-                  ' This new requirements for sacrud >= 0.1.1 version"'
-        print '\033[93m' + message + '\033[0m'
-        raise Exception(message)
-    return settings[name]
+from sacrud.pyramid_ext.breadcrumbs import breadcrumbs
+from sacrud.common.paginator import get_paginator
+from sacrud.common.pyramid_helpers import get_settings_param
 
 
 def get_table(tname, request):
-    """ Return table by table name.
+    """ Return table by table name from sacrud.models in settings.
     """
+    # TODO: write test
     # convert values of models dict to flat list
-    tables = itertools.chain(*get_config(request, 'sacrud.models').values())
+    tables = itertools.chain(*get_settings_param(request, 'sacrud.models').values())
     return filter(lambda table: (table.__tablename__).
                   lower() == tname.lower(), tables)[0]
 
 
 def get_relationship(tname, request):
+    # TODO: write test
     obj = get_table(tname, request)
     # Build a list of only relationship properties
     relation_properties = filter(
@@ -73,7 +40,7 @@ def get_relationship(tname, request):
 
 @view_config(route_name='sa_home', renderer='/sacrud/home.jinja2')
 def sa_home(request):
-    return {'tables': get_config(request, 'sacrud.models')}
+    return {'tables': get_settings_param(request, 'sacrud.models')}
 
 
 @view_config(route_name='sa_list', renderer='/sacrud/list.jinja2')
@@ -81,55 +48,61 @@ def sa_list(request):
     tname = request.matchdict['table']
     order_by = request.params.get('order_by', False)
     args = [DBSession, get_table(tname, request)]
+
     if order_by:
         args.append(order_by)
-    resp = action.index(*args)
+    resp = action.list(*args, paginator=get_paginator(request))
     return {'sa_crud': resp, 'breadcrumbs': breadcrumbs(tname, 'sa_list')}
 
 
-@view_config(route_name='sa_create', renderer='/sacrud/create.jinja2')
-def sa_create(request):
-    tname = request.matchdict['table']
-    table = get_table(tname, request)
-    fk_list = sa.inspect(DBSession.bind).get_foreign_keys(tname)
-    fk_list = map(lambda x: x['constrained_columns'], fk_list)  # only col name
-    fk_list = list(itertools.chain(*fk_list))  # list of lists to flat list
+class CRUD(object):
+    def __init__(self, request):
+        self.request = request
+        self.tname = request.matchdict['table']
+        self.table = get_table(self.tname, self.request)
+        self.relationships = get_relationship(self.tname, self.request)
+        self.id = request.matchdict.get('id')
+        self.params = request.params.dict_of_lists()
 
-    if 'form.submitted' in request.params:
-        action.create(DBSession, table,
-                      request.params.dict_of_lists())
-        return HTTPFound(location=request.route_url('sa_list', table=tname))
-    resp = action.create(DBSession, table)
-    rel = get_relationship(tname, request)
-    return {'sa_crud': resp, 'rel': rel, 'fk_list': fk_list,
-            'breadcrumbs': breadcrumbs(tname, 'sa_create')}
+    def flash_message(self, message):
+        if hasattr(self.request, 'session'):
+            self.request.session.flash(message)
 
+    @view_config(route_name='sa_create', renderer='/sacrud/create.jinja2')
+    def sa_create(self):
+        if 'form.submitted' in self.request.params:
+            action.create(DBSession, self.table,
+                          self.request.params.dict_of_lists())
+            self.flash_message("You created new object of %s" % self.tname)
+            return HTTPFound(location=self.request.route_url('sa_list', table=self.tname))
+        resp = action.create(DBSession, self.table)
+        return {'sa_crud': resp,
+                'relationships': self.relationships,
+                'breadcrumbs': breadcrumbs(self.tname, 'sa_create')}
 
-@view_config(route_name='sa_read', renderer='/sacrud/read.jinja2')
-def sa_read(request):
-    tname = request.matchdict['table']
-    id = request.matchdict['id']
-    resp = action.read(DBSession, get_table(tname, request), id)
-    return {'sa_crud': resp,
-            'breadcrumbs': breadcrumbs(tname, 'sa_read', id=id)}
+    @view_config(route_name='sa_read', renderer='/sacrud/read.jinja2')
+    def sa_read(self):
+        resp = action.read(DBSession,
+                           get_table(self.tname, self.request), self.id)
+        return {'sa_crud': resp,
+                'breadcrumbs': breadcrumbs(self.tname, 'sa_read', id=self.id)}
 
+    @view_config(route_name='sa_update', renderer='/sacrud/create.jinja2')
+    def sa_update(self):
+        if 'form.submitted' in self.request.params:
+            action.update(DBSession, self.table, self.id, self.params)
+            self.flash_message("You updated object of %s" % self.tname)
+            return HTTPFound(location=self.request.route_url('sa_list', table=self.tname))
+        resp = action.update(DBSession, self.table, self.id)
+        return {'sa_crud': resp,
+                'relationships': self.relationships,
+                'breadcrumbs': breadcrumbs(self.tname, 'sa_update', id=self.id)}
 
-@view_config(route_name='sa_update', renderer='/sacrud/create.jinja2')
-def sa_update(request):
-    id = request.matchdict['id']
-    tname = request.matchdict['table']
-    table = get_table(tname, request)
-    fk_list = sa.inspect(DBSession.bind).get_foreign_keys(tname)
-    fk_list = map(lambda x: x['constrained_columns'], fk_list)  # only col name
-    fk_list = list(itertools.chain(*fk_list))  # list of lists to flat list
-
-    if 'form.submitted' in request.params:
-        action.update(DBSession, table, id, request.params.dict_of_lists())
-        return HTTPFound(location=request.route_url('sa_list', table=tname))
-    resp = action.update(DBSession, table, id)
-    rel = get_relationship(tname, request)
-    return {'sa_crud': resp, 'rel': rel, 'fk_list': fk_list,
-            'breadcrumbs': breadcrumbs(tname, 'sa_update', id=id)}
+    @view_config(route_name='sa_delete')
+    def sa_delete(self):
+        action.delete(DBSession, self.table, self.id)
+        self.flash_message("You have removed object of %s" % self.tname)
+        return HTTPFound(location=self.request.route_url('sa_list', table=self.tname))
 
 
 @view_config(route_name='sa_paste', renderer='/sacrud/list.jinja2')
@@ -146,16 +119,9 @@ def sa_paste(request):
     return HTTPFound(location=request.route_url('sa_list', table=tname))
 
 
-@view_config(route_name='sa_delete')
-def sa_delete(request):
-    tname = request.matchdict['table']
-    id = request.matchdict['id']
-    action.delete(DBSession, get_table(tname, request), id)
-    return HTTPFound(location=request.route_url('sa_list', table=tname))
-
-
 @view_config(route_name='sa_union_fields', renderer='/sacrud/compare.jinja2')
 def sa_union_fields(request):
+    # XXX: experimental future
     tname = request.matchdict['table']
     table = get_table(tname, request)
     pk = action.get_pk(table)
@@ -183,7 +149,7 @@ def sa_union_fields(request):
     args = [DBSession, table]
     if order_by:
         args.append(order_by)
-    resp = action.index(*args)
+    resp = action.list(*args)
 
     sa_checked_row = request.POST.get('sa_checked_row', None).split(",")
     if '' not in sa_checked_row:
